/*******************************************************************
 *
 * check_ntload plugin
 * Copyright (c) 2000 Jan Christian Kaldestad
 *
 *
 * Description:
 * 	 Checks the processor load average on NT servers.
 *	 Also finds the process with highest cpu usage.
 *
 ******************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <sys/errno.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>


#include "../../Common/queries.h"
#include "../../Common/kvbuffer.h"
#include "util.h"



const char* PLUGIN_NAME = "check_ntload";



bool CheckCmdLine(int argc, char **argv, KVBuffer& sbuffer)
{
	if (argc < 9)
        {
		cout << "Incorrect arguments supplied\n\n";
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << "Note:\n";
    	cout << " <interval>  = time interval in minutes used to measure the cpu load average," << endl;
    	cout << "               must be a number between 0 and 15 minutes" << endl;
    	cout << " <wload>     = cpu load limit to result in a WARNING message," << endl;
    	cout << "               must be a number between 0 and 100" << endl;
    	cout << " <cload>     = cpu load limit to result in a CRITICAL message," << endl;
    	cout << "               must be a number between 0 and 100" << endl;

    	return false;
    }

    int interval1 = atoi(argv[3]);
    int wload1	  = atoi(argv[4]);
    int cload1	  = atoi(argv[5]);
    int interval2 = atoi(argv[6]);
    int wload2	  = atoi(argv[7]);
    int cload2	  = atoi(argv[8]);

    if( (interval1<0) || (interval1>MAX_INTERVAL))
    {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <interval1> must be a number between 0 and 15 minutes" << endl;
    	return STATE_UNKNOWN;
    }

    if( (wload1<0) || (wload1>100) || (cload1<0) || (cload1>100) )
    {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <wload1> and <cload1> must be a number between 0 and 100" << endl;
    	return false;
      }

    if(wload1>cload1)
    {
    	cout << "Inconsitstence in parameters: \"warning load\" greater than \"critical load\"" << endl;
    	return false;
    }

    if( (interval2<0) || (interval2>MAX_INTERVAL))
    {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <interval2> must be a number between 0 and 15 minutes" << endl;
    	return false;
    }

    if( (wload2<0) || (wload2>100) || (cload2<0) || (cload2>100) )
    {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <wload2> and <cload2> must be a number between 0 and 100" << endl;
    	return false;
      }

    if(wload2>cload2)
    {
    	cout << "Inconsistencies in parameters: \"warning load\" greater than \"critical load\"" << endl;
    	return false;
    }

    sbuffer.addKeyIntValue("id", CPU_LOAD_QUERY);
    sbuffer.addKeyIntValue("i1", interval1);
    sbuffer.addKeyIntValue("wload1", wload1);
    sbuffer.addKeyIntValue("cload1", cload1);
    sbuffer.addKeyIntValue("i2", interval2);
    sbuffer.addKeyIntValue("wload2", wload2);
    sbuffer.addKeyIntValue("cload2", cload2);

    return true;
}

int CheckLoad(int cpuLoad, int wload, int cload)
{
	if (cpuLoad < wload)
		return STATE_OK;

	if (cpuLoad < cload)
		return STATE_WARNING;

	return STATE_CRITICAL;
}

bool GetStringAndState(int argc, char** argv, KVBuffer& sbuf, KVBuffer& rbuf, int& state, char* msg)
{
	if( rbuf.getIntValue("id") != CPU_LOAD_QUERY )
	{
		cout << "Error in server reply" << endl;
		return STATE_UNKNOWN;
	}

	int cpuLoad1 = rbuf.getIntValue("l1");
	int cpuLoad2 = rbuf.getIntValue("l2");

        int prosName = rbuf.getIntValue("pname");
	int prosLoad = rbuf.getIntValue("pload");

	int interval1 = sbuf.getIntValue("i1");
	int interval2 = sbuf.getIntValue("i2");

	state = STATE_OK;

        int ret1 = CheckLoad(cpuLoad1, sbuf.getIntValue("wload1"), sbuf.getIntValue("cload1"));
        int ret2 = CheckLoad(cpuLoad1, sbuf.getIntValue("wload2"), sbuf.getIntValue("cload2"));

	if(ret1 > ret2)
        {
		state = ret1;
  	}
	else
	{
        	state = ret2;
	}

	switch(state)
	{
		case STATE_OK:
                	sprintf(msg, "CPU Load OK (%dm: %d%s) (%dm: %d%s)", interval1, cpuLoad1, "%%", interval2, cpuLoad2, "%%");
			break;
		default:
			if(prosLoad > 0)
			{
                		sprintf(msg, "CPU Load HIGH (%dm: %d%s) (%dm: %d%s) Process: %s using %d%s", interval1, cpuLoad1, "%%", interval2, cpuLoad2, "%%", prosName, prosLoad, "%%");
			}
                        else
                        {
                		sprintf(msg, "CPU Load HIGH (%dm: %d%s) (%dm: %d%s) caused by <Unknown process>", interval1, cpuLoad1, "%%", interval2, cpuLoad2, "%%");
                        }
                        break;
	}

        return true;
}

